@@ -14,12 +14,10 @@ module.exports = function (api) {
   const envOptsNoTargets = {
     loose: true,
     shippedProposals: true,
+    modules: false,
   };
   const envOpts = Object.assign({}, envOptsNoTargets);
 
-  const compileDynamicImport =
-    env === "test" || env === "development" || env === "test-legacy";
-
   let convertESM = true;
   let ignoreLib = true;
   let includeRegeneratorRuntime = false;
@@ -103,6 +101,10 @@ module.exports = function (api) {
       .filter(Boolean)
       .map(normalize),
     presets: [
+      [
+        "@babel/preset-typescript",
+        { onlyRemoveTypeImports: true, allowDeclareFields: true },
+      ],
       ["@babel/env", envOpts],
       ["@babel/preset-flow", { allowDeclareFields: true }],
     ],
@@ -111,9 +113,8 @@ module.exports = function (api) {
         "@babel/proposal-object-rest-spread",
         { useBuiltIns: true, loose: true },
       ],
-      compileDynamicImport ? dynamicImportUrlToPath : null,
-      compileDynamicImport ? "@babel/plugin-proposal-dynamic-import" : null,
 
+      convertESM ? "@babel/proposal-export-namespace-from" : null,
       convertESM ? "@babel/transform-modules-commonjs" : null,
     ].filter(Boolean),
     overrides: [
@@ -159,45 +160,3 @@ module.exports = function (api) {
 
   return config;
 };
-
-// !!! WARNING !!! Hacks are coming
-
-// import() uses file:// URLs for absolute imports, while require() uses
-// file paths.
-// Since this isn't handled by @babel/plugin-transform-modules-commonjs,
-// we must handle it here.
-// However, fileURLToPath is only supported starting from Node.js 10.
-// In older versions, we can remove the pathToFileURL call so that it keeps
-// the original absolute path.
-// NOTE: This plugin must run before @babel/plugin-transform-modules-commonjs,
-// and assumes that the target is the current node version.
-function dynamicImportUrlToPath({ template, env }) {
-  const currentNodeSupportsURL =
-    !!require("url").pathToFileURL && env() !== "test-legacy"; // test-legacy is run on legacy node versions without pathToFileURL support
-  if (currentNodeSupportsURL) {
-    return {
-      visitor: {
-        CallExpression(path) {
-          if (path.get("callee").isImport()) {
-            path.get("arguments.0").replaceWith(
-              template.expression.ast`
-              require("url").fileURLToPath(${path.node.arguments[0]})
-            `
-            );
-          }
-        },
-      },
-    };
-  } else {
-    // TODO: Remove in Babel 8 (it's not needed when using Node 10)
-    return {
-      visitor: {
-        CallExpression(path) {
-          if (path.get("callee").isIdentifier({ name: "pathToFileURL" })) {
-            path.replaceWith(path.get("arguments.0"));
-          }
-        },
-      },
-    };
-  }
-}
