@@ -278,7 +278,7 @@ export default class ExpressionParser extends LValParser {
       node.operator = operator;
 
       if (this.match(tt.eq)) {
-        node.left = this.toAssignable(left);
+        node.left = this.toAssignable(left, /* isLHS */ true);
         refExpressionErrors.doubleProto = -1; // reset because double __proto__ is valid in assignment expression
       } else {
         node.left = left;
@@ -288,7 +288,7 @@ export default class ExpressionParser extends LValParser {
         refExpressionErrors.shorthandAssign = -1; // reset because shorthand default was used correctly
       }
 
-      this.checkLVal(left, undefined, undefined, "assignment expression");
+      this.checkLVal(left, "assignment expression");
 
       this.next();
       node.right = this.parseMaybeAssign();
@@ -539,7 +539,7 @@ export default class ExpressionParser extends LValParser {
     refExpressionErrors: ?ExpressionErrors,
   ): N.Expression {
     if (update) {
-      this.checkLVal(node.argument, undefined, undefined, "prefix operation");
+      this.checkLVal(node.argument, "prefix operation");
       return this.finishNode(node, "UpdateExpression");
     }
 
@@ -552,7 +552,7 @@ export default class ExpressionParser extends LValParser {
       node.operator = this.state.value;
       node.prefix = false;
       node.argument = expr;
-      this.checkLVal(expr, undefined, undefined, "postfix operation");
+      this.checkLVal(expr, "postfix operation");
       this.next();
       expr = this.finishNode(node, "UpdateExpression");
     }
@@ -842,7 +842,6 @@ export default class ExpressionParser extends LValParser {
     nodeForExtra?: ?N.Node,
   ): $ReadOnlyArray<?N.Expression> {
     const elts = [];
-    let innerParenStart;
     let first = true;
     const oldInFSharpPipelineDirectBody = this.state.inFSharpPipelineDirectBody;
     this.state.inFSharpPipelineDirectBody = false;
@@ -875,12 +874,6 @@ export default class ExpressionParser extends LValParser {
         }
       }
 
-      // we need to make sure that if this is an async arrow functions,
-      // that we don't allow inner parens inside the params
-      if (this.match(tt.parenL) && !innerParenStart) {
-        innerParenStart = this.state.start;
-      }
-
       elts.push(
         this.parseExprListItem(
           false,
@@ -891,11 +884,6 @@ export default class ExpressionParser extends LValParser {
       );
     }
 
-    // we found an async arrow function so let's not allow any inner parens
-    if (possibleAsyncArrow && innerParenStart && this.shouldParseAsyncArrow()) {
-      this.unexpected();
-    }
-
     this.state.inFSharpPipelineDirectBody = oldInFSharpPipelineDirectBody;
 
     return elts;
@@ -1421,12 +1409,6 @@ export default class ExpressionParser extends LValParser {
     ) {
       this.expressionScope.validateAsPattern();
       this.expressionScope.exit();
-      for (const param of exprList) {
-        if (param.extra && param.extra.parenthesized) {
-          this.unexpected(param.extra.parenStart);
-        }
-      }
-
       this.parseArrowExpression(arrowNode, exprList, false);
       return arrowNode;
     }
@@ -2056,7 +2038,7 @@ export default class ExpressionParser extends LValParser {
     params: N.Expression[],
     trailingCommaPos: ?number,
   ): void {
-    node.params = this.toAssignableList(params, trailingCommaPos);
+    node.params = this.toAssignableList(params, trailingCommaPos, false);
   }
 
   parseFunctionBodyAndFinish(
@@ -2126,9 +2108,9 @@ export default class ExpressionParser extends LValParser {
           if (this.state.strict && node.id) {
             this.checkLVal(
               node.id,
+              "function name",
               BIND_OUTSIDE,
               undefined,
-              "function name",
               undefined,
               strictModeChanged,
             );
@@ -2157,14 +2139,13 @@ export default class ExpressionParser extends LValParser {
     isArrowFunction: ?boolean,
     strictModeChanged?: boolean = true,
   ): void {
-    // $FlowIssue
-    const nameHash: {} = Object.create(null);
-    for (let i = 0; i < node.params.length; i++) {
+    const checkClashes = new Set();
+    for (const param of node.params) {
       this.checkLVal(
-        node.params[i],
-        BIND_VAR,
-        allowDuplicates ? null : nameHash,
+        param,
         "function parameter list",
+        BIND_VAR,
+        allowDuplicates ? null : checkClashes,
         undefined,
         strictModeChanged,
       );
