@@ -8,12 +8,18 @@ require("core-js/modules/es.array.flat-map");
 
 require("core-js/modules/es.array.iterator");
 
+require("core-js/modules/es.array.reduce");
+
+require("core-js/modules/es.array.reduce-right");
+
 require("core-js/modules/es.array.sort");
 
 require("core-js/modules/es.array.unscopables.flat");
 
 require("core-js/modules/es.array.unscopables.flat-map");
 
+require("core-js/modules/es.math.hypot");
+
 require("core-js/modules/es.object.define-getter");
 
 require("core-js/modules/es.object.define-setter");
